////////////////////////////////////////
// PurchaseInformation Implementation	//
//																	  //
////////////////////////////////////////

#pragma warning (disable : 4786)

#ifndef  PRODUCTINFORMATION_CLASS
#include "ProductInformation.h"
#endif

using namespace Products;

// default ctor
ProductInformation::ProductInformation() throw()
  :  m_ProductCode("Unknow"),
     m_Name ("Unknow"),
     m_PricePerUnit(0.00),
     m_TaxableStatus (true),
     m_QuantityOnHand (0)
{}

ProductInformation::ProductInformation(Common::UPCCode  Code, Common::String Name,
                                       Common::Money PricePerUnit, bool TaxableStatus,
                                       Common::Quantity QuantityOnHand) throw()
  :  m_ProductCode(Code),
     m_Name (Name),
     m_PricePerUnit(PricePerUnit),
     m_TaxableStatus (TaxableStatus),
     m_QuantityOnHand (QuantityOnHand)
{}

// dtor
ProductInformation::~ProductInformation() throw()
{
  // empty
}


// Copy constructor
ProductInformation::ProductInformation(const ProductInformation &other) throw()
{
  m_ProductCode    = other.m_ProductCode;
  m_Name           = other.m_Name;
  m_PricePerUnit   = other.m_PricePerUnit;
  m_TaxableStatus  = other.m_TaxableStatus;
  m_QuantityOnHand = other.m_QuantityOnHand;
}

ProductInformation &ProductInformation::operator= (const ProductInformation &other) throw()
{
  if (this == &other)
    return *this;

  m_ProductCode    = other.m_ProductCode;
  m_Name           = other.m_Name;
  m_PricePerUnit   = other.m_PricePerUnit;
  m_TaxableStatus  = other.m_TaxableStatus;
  m_QuantityOnHand = other.m_QuantityOnHand;

  return *this;
}

ProductInformation & ProductInformation::SetProductCode(Common::UPCCode Code) throw()
{
  m_ProductCode = Code;
  return *this;
}


ProductInformation & ProductInformation::SetName(Common::String Name) throw()
{
  m_Name = Name;
  return *this;
}

ProductInformation & ProductInformation::SetPricePerUnit(Common::Money PricePerUnit) throw(Common::OutOfRangeException)
{
  if (PricePerUnit < 0.0 )
    throw Common::OutOfRangeException();
  else
    m_PricePerUnit = PricePerUnit;

  return *this;
}

ProductInformation & ProductInformation::SetTaxableStatus(bool TaxableStatus) throw(Common::NegativeValueException)
{
  if ( (TaxableStatus != true) || (TaxableStatus != false) )
    throw Common::NegativeValueException();
  else
    m_TaxableStatus = TaxableStatus;

  return *this;
}

ProductInformation & ProductInformation::SetQuantityOnHand(Common::Quantity QuantityOnHand) throw()
{
  m_QuantityOnHand = QuantityOnHand;
  return *this;
}