////////////////////////////////////////
// ProductInformation Implementation	//
//																	  //
////////////////////////////////////////

#pragma once

#ifndef PRODUCTINFORMATION_CLASS
#define PRODUCTINFORMATION_CLASS

#ifndef GENERIC_CLASS
#include "c:\scs\common\generic.h"
#endif

#include "ProductException.h"


namespace Common
{
  class OutOfRangeException;
  class NegativeValueException;
  class EmptyStringException;
}

namespace Products
{

  class ProductInformation
    {
    public:
        ProductInformation() throw();
        ProductInformation(Common::UPCCode  Code, Common::String Name,
                           Common::Money PricePerUnit, bool TaxableStatus,
                           Common::Quantity QuantityOnHand) throw();
        ~ProductInformation() throw();

        ProductInformation(const ProductInformation &other) throw();          
        ProductInformation &operator= (const ProductInformation &other) throw();
        
        // Data Access Functions
        //
        Common::UPCCode  GetProductCode(void)    const throw() {return m_ProductCode;}
        Common::String   GetName(void)           const throw() {return m_Name;}
        Common::Money    GetPricePerUnit(void)   const throw() {return m_PricePerUnit;}
        bool             GetTaxableStatus(void)  const throw() {return m_TaxableStatus;}
        Common::Quantity GetQuantityOnHand(void) const throw() {return m_QuantityOnHand;}

        ProductInformation & SetProductCode(Common::UPCCode Code) throw();
        ProductInformation & SetName(Common::String Name) throw();
        ProductInformation & SetPricePerUnit(Common::Money PricePerUnit) throw(Common::OutOfRangeException);
        ProductInformation & SetTaxableStatus(bool TaxableStatus) throw(Common::NegativeValueException);
        ProductInformation & SetQuantityOnHand(Common::Quantity QuantityOnHand) throw();      

    private:
        // Member Data
        //
        Common::UPCCode  m_ProductCode;
        Common::String   m_Name;
        Common::Money    m_PricePerUnit;
        bool             m_TaxableStatus;
        Common::Quantity m_QuantityOnHand;
        
    };
}

#endif