// EventServer.cpp

#pragma warning (disable : 4786)

#include "EventServer.h"

// ** Reactor Implementation **
Reactor & Reactor::GetInstance ()
{
  static Reactor as_Reactor;
  return as_Reactor;
}

// Register
void Reactor::RegisterHandler(const std::type_info & p_EventTypeInfo,
                              EventHandler & p_EventHandler)
{
  //cout << "Reactor:: " << typeid(p_EventHandler).name() << " has registered event " <<  p_EventTypeInfo.name() << endl;
  std::string Key = std::string(p_EventTypeInfo.name());
  std::multimap<std::string, EventHandler *>::value_type
     a_NewValue( Key, &p_EventHandler );
  m_SubscriberCollection.insert(a_NewValue);

  //  Why not???
  //  std::multimap<std::string, EventHandler *>::pair<const std::string, EventHandler *>
  //     a_NewValue( Key, &p_EventHandler );
  //  m_SubscriberCollection.insert(a_NewValue);
  

  // ** Doesn't work this way because multimap doesn't overload operator[]
  //  m_SubscriberCollection[Key] = &p_EventHandler;
}


// Remove
void Reactor::RemoveHandler(const std::type_info & p_EventTypeInfo,
                            EventHandler & p_EventHandler)
{
  // Get a range of all possible EventHandler with same class name
  std::pair< SubscriberMap::iterator, SubscriberMap::iterator >
      a_Range = m_SubscriberCollection.equal_range(p_EventTypeInfo.name());

  SubscriberMap::iterator a_CurrentIter = a_Range.first;
  SubscriberMap::iterator a_EndIter     = a_Range.second;

  while ( a_CurrentIter != a_EndIter )
  {
    // Get exact same type of EventHandler and then delete
    if ( a_CurrentIter->second == & p_EventHandler ) // second contains a pointer
    {
      SubscriberMap::iterator a_NextIter = a_CurrentIter;
      ++a_NextIter;   // need to step forward before a_CurrentIter is deleted
      //cout << "Reactor:: unregisters event " << a_CurrentIter->first  << " from " 
      //     << typeid(p_EventHandler).name() << endl;
      m_SubscriberCollection.erase(a_CurrentIter);
      a_CurrentIter = a_NextIter;   // reassign pointer to next location 
    }
    else  // if pointer mismatch
      ++ a_CurrentIter;
  }
}


// Dispatch Event
void Reactor::DispatchEvent(const Event & p_Event)
{
  // Get a range of ALL subscribers
  std::pair< SubscriberMap::iterator, SubscriberMap::iterator >
      a_Range = m_SubscriberCollection.equal_range(typeid(p_Event).name());

  SubscriberMap::iterator a_CurrentIter = a_Range.first;
  SubscriberMap::iterator a_EndIter     = a_Range.second;

  while ( a_CurrentIter != a_EndIter )
  {
    SubscriberMap::iterator a_TargetIter = a_CurrentIter;
    ++a_CurrentIter;
    // no need to use if_check because all items within the range are subscribers
    ( (*a_TargetIter).second )->HandleEvent(p_Event);
    // <Polymorphism>
    // ** EventHandlerPtr->HandelEvent() **

    // a_CurrentIter->second->HandleEvent(p_Event);    
  }
}

    









