// EvenServer.h

#pragma warning (disable : 4786)

#ifndef EVENTSERVER_CLASS
#define EVENTSERVER_CLASS

#include <iostream>
#include <typeinfo>
#include <string>
#include <map>
// #include <utility>

using std::cout;
using std::endl;


// ** Event
class Event
{
  public:
    Event() {}
    virtual ~Event() = 0 {}
  private:
    Event(const Event &other);
    Event & operator= (const Event &other);    
};

// ** EventHandler
class EventHandler
{
  public:
    // Not necessary for standard since 
    // it is crated to enforce Polymorphism of HandleEvent function
    
    // Important function
    virtual void HandleEvent (const Event & p_Event) = 0;

};

// ** Reactor
class Reactor
{
  public:
    // Singleton design
    ~Reactor() {}
    static Reactor & GetInstance ();

    // functions to Register, Remove, and Dispatch events
    void RegisterHandler(const std::type_info & p_EventTypeInfo,
                         EventHandler & p_EventHandler);

    void RemoveHandler(const std::type_info & p_EventTypeInfo,
                       EventHandler & p_EventHandler);

    // trigger an Event, therefore pass in an Event
    void DispatchEvent(const Event & p_Event);
  

  private:
    Reactor() {}
    typedef std::multimap<std::string, EventHandler *> SubscriberMap;
    std::multimap< std::string, EventHandler * > m_SubscriberCollection;

    // No implementation
    Reactor(const Reactor & p_Reactor);
    Reactor & operator= (const Reactor & p_Reactor);
};






#endif