// Factory.h

#pragma warning (disable : 4786)

#ifndef FACTORY_CLASS
#define FACTORY_CLASS

#include <map>
#include <typeinfo>

#ifndef GENERIC_CLASS
#include "c:\scs\common\generic.h"
#endif

using std::cout;
using std::endl;

//  Architecture:
//  Key --> Factory --> Builder --> Product

template < class KEY, class BUILDER, class PRODUCT >
class Factory
{
  public:
    static Factory<KEY, BUILDER, PRODUCT> & GetInstance(void);

    void AddBuilder ( KEY p_keyData, BUILDER * p_builderPtr );
    void RemoveBuilder ( KEY p_keyData );
    
    PRODUCT * BuildProduct ( KEY p_rawData );

  private:
    // ctor in private section
    Factory() {}

    // STL <map>
    std::map < KEY, BUILDER * > m_builderMap;
};

template <class KEY, class BUILDER, class PRODUCT>
Factory<KEY, BUILDER, PRODUCT> & Factory<KEY, BUILDER, PRODUCT>::GetInstance(void)
{
  static Factory<KEY, BUILDER, PRODUCT> as_Factory;
  return as_Factory;
}

template <class KEY, class BUILDER, class PRODUCT>
void Factory<KEY, BUILDER, PRODUCT>::AddBuilder ( KEY p_keyData, BUILDER * p_builderPtr )
{
  bool exist = 0;
  std::map < KEY, BUILDER * >::iterator i;
  for ( i = m_builderMap.begin(); i != m_builderMap.end(); i++)
  {
    if ( i->first == p_keyData ) exist = 1;
  }
  if ( !exist )
  {
    m_builderMap[p_keyData] = p_builderPtr;
    // cout << typeid(*p_builderPtr).name() << " registers for keyword \"" << p_keyData << "\"" << endl;
  }
}

template <class KEY, class BUILDER, class PRODUCT>
void Factory<KEY, BUILDER, PRODUCT>::RemoveBuilder ( KEY p_keyData )
{
  m_builderMap.erase( p_keyData );
}

template <class KEY, class BUILDER, class PRODUCT>
PRODUCT * Factory<KEY, BUILDER, PRODUCT>::BuildProduct ( KEY p_rawData )
{
  BUILDER * a_builderPtr = m_builderMap[ p_rawData ];
  if ( a_builderPtr != 0 ) 
  {
    return a_builderPtr->BuildProduct();  
  }
  else 
  {
    cout << "Invalid Event ID ... and Event Pointer is set to NULL" << endl;
    return 0;
  }
}



#endif