////////////////////////////////////////////
// Reference Counting Pointer Classes   	//
//																	      //
////////////////////////////////////////////

#ifndef REFERENCECOUNTING_CLASS
#define REFERENCECOUNTING_CLASS

// base class 
class RefCounted
{
  public:
    RefCounted() : m_count(0) {}
    RefCounted( const RefCounted &other) : m_count(0) {}
    virtual ~RefCounted() = 0 {} 
    RefCounted & operator= (const RefCounted &other) {return *this;}
    void Attach() {++m_count;}
    void Detach() 
      {
        if ( m_count>0 ) --m_count;
        if ( m_count==0 ) delete this;  // destroy the class
      }
    unsigned long GetCount (void) const { return m_count; }

  private:
    unsigned long m_count;
};


template <class T>  // T is derive class of RefCounted
class RCPointer
{
  public:
    RCPointer() : m_targetPtr(0) {}
    RCPointer(T * otherPtr);
    RCPointer ( const RCPointer &other);
    ~RCPointer();

    RCPointer & operator= (const RCPointer &other);
    RCPointer & operator= (T *otherPtr);

    operator T * (void) const { return m_targetPtr; }
    T * operator-> (void) const { return m_targetPtr;}

  private:
    T * m_targetPtr;
};


// ** RCPointer Template Implementation
template <class T>
RCPointer<T>::RCPointer(T * otherPtr)
: m_targetPtr(otherPtr)
{
  if (m_targetPtr != 0)
    m_targetPtr->Attach();
}


template <class T>
RCPointer<T>::RCPointer(const RCPointer &other)
: m_targetPtr(other.m_targetPtr)
{
  if (m_targetPtr != 0)
    m_targetPtr->Attach();
}


template <class T>
RCPointer<T>::~RCPointer()
{
  if (m_targetPtr != 0)         // T is Transaction instance
    m_targetPtr->Detach();      // when one transaction is complete, detach it from section
}


template <class T>
RCPointer<T> & RCPointer<T>::operator= (const RCPointer &other)
{
  if ( (this == &other) && (other.m_targetPtr == m_targetPtr) )
    return *this;

  if (m_targetPtr != 0)
    m_targetPtr->Detach();

  m_targetPtr = other.m_targetPtr;

  if (m_targetPtr != 0)
    m_targetPtr->Attach();

  return *this;
}


template <class T>
RCPointer<T> & RCPointer<T>::operator= (T *otherPtr)
{
  if (this == otherPtr)
    return *this;

  if (m_targetPtr != 0)
    m_targetPtr->Detach();

  m_targetPtr = otherPtr;

  if (m_targetPtr != 0)
    m_targetPtr->Attach();

  return *this;
}


#endif